// Fill out your copyright notice in the Description page of Project Settings.


#include "VisionLinkTutorial.h"

#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "EndlessVendetta/GadgetSystem/GadgetManager.h"
#include "Kismet/KismetMathLibrary.h"

void AVisionLinkTutorial::BeginPlay()
{
	Super::BeginPlay();
	SpawnEnemiesAndSetupDefaults();
	SpawnNewWaypoint(GoToAreaWaypointLoc, GoToAreaWaypointDesc);
	SetActorTickInterval(0.2);
}

void AVisionLinkTutorial::Tick(float DeltaSeconds)
{
	Super::Tick(DeltaSeconds);

	if (!PlayerInTutorialZone)
	{
		if (PlayerWasInTutZone) SpawnNewWaypoint(GoToAreaWaypointLoc, GoToAreaWaypointDesc);
		PlayerWasInTutZone = false;
		return;
	}

	if (!PlayerWasInTutZone) SpawnNewWaypoint(LookAtTargetWaypointLoc, LookAtTargetWaypointDesc);
	PlayerWasInTutZone = true;
	
	// Check if players recon gadget (vision link) is in active
	APawn* PlayerPawn = GetWorld()->GetFirstPlayerController()->GetPawn();
	AEndlessVendettaCharacter* EndlessVendettaCharacter = Cast<AEndlessVendettaCharacter>(PlayerPawn);
	AGadgetManager* GadgetManager = EndlessVendettaCharacter->GadgetManager;
	if (!GadgetManager->ReconInUse()) return;

	// Check if the player is roughly looking at their target
	FRotator LookAtRot = UKismetMathLibrary::FindLookAtRotation(PlayerPawn->GetActorLocation(), TargetLoc);
	FRotator PlayersRotator = PlayerPawn->GetActorRotation();
	// Keep all yaw values between -180 and 180 to simplify the comparison
	float DesiredtYaw = LookAtRot.Yaw - (LookAtRot.Yaw > 180 ? 360 : 0);
	float PlayersYaw = PlayersRotator.Yaw - (PlayersRotator.Yaw > 180 ? 360 : 0);
	// Do nothing if players yaw is off by more than 45 degrees
	if (FMath::Abs(DesiredtYaw - PlayersYaw) > 45) return;

	
	SpawnNewWaypoint(ResultWaypointLoc, ResultWaypointDesc);
	FTimerHandle TimerHandle;
	GetWorld()->GetTimerManager().SetTimer(TimerHandle, this, &AVisionLinkTutorial::FinishedTutorial, 7.5);
	SetActorTickEnabled(false);
}



void AVisionLinkTutorial::DestroyTutorial()
{
	if (IsValid(WaypointActor)) WaypointActor->Destroy();
	DespawnEnemies();
	Super::DestroyTutorial();
}

void AVisionLinkTutorial::FinishedTutorial()
{
	if (IsValid(WaypointActor)) WaypointActor->Destroy();
	CompletedGadgetTutorial.Broadcast();
}




